var Report = function()
{
	var report = {};
	var debug = 0;
	
	var PLOT_WIDTH = 700;
	var PLOT_HEIGHT = 550;
	var PLOT_SCALE = 2;
	
	var periodIsValid = true;
	var xGuide;
	var yGuide;
	
	report.initPage = function()
	{
		var frm = document.forms['reggie'];

		// Step 1
		Events.addEventHandler('fromdate', 'blur', report.dateOnChange)
		Events.addEventHandler('todate', 'blur', report.dateOnChange)
		Events.addEventHandler('step-1', 'wizard-validate', report.validateStep1);
		
		// Step 2
		Events.addEventHandler('printButton', 'click', report.printVersion);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('gocreate', Wizard.goRegister);
	
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', report.submit);
		
		Doc.show('step-1');
		Doc.show('gocreate');
		frm.fromdate.focus();

		// Load sites
		var url = '../ScanBQuarterMonthReport.servlet?ID='+App.getSessionId();
		url += '&cmd=GetSites';		
		Doc.addClass('sites', 'list-loading');
		frm.sites[0] = new Option('loading...');
		Wizard.asyncJsonRequest(url, report.sitesLoaded);
	}
	
	
	report.sitesLoaded = function(response)
	{
		Doc.removeClass('sites', 'list-loading');
		var frm = document.forms['reggie'];
		var sites = response.sites;
		frm.sites.length = 0;
		frm.sites[0] = new Option('All sites together', '');
		for (var i = 0; i < sites.length; i++)
		{
			var s = sites[i];
			var option = new Option(s.name, s.prefix);
			frm.sites[frm.sites.length] = option;
		}
	}

	report.dateOnChange = function()
	{
		var frm = document.forms['reggie'];		
		var fromDate = frm.fromdate.value;
		var toDate = frm.todate.value;
		
		periodIsValid = false;
		Wizard.setInputStatus('period');

		if (fromDate != '')
		{
			fromDate = Reggie.autoFillDate(fromDate, true);
			frm.fromdate.value = fromDate;			
			if (!Dates.isDate(fromDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('period', 'invalid', 'Not a valid from-date');
				return;
			}
		}

		if (toDate != '')
		{
			toDate = Reggie.autoFillDate(toDate, true);
			frm.todate.value = toDate;			
			if (!Dates.isDate(toDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('period', 'invalid', 'Not a valid to-date');
				return;
			}
		}
		
		if (toDate && fromDate)
		{
			if (fromDate > toDate)
			{
				Wizard.setInputStatus('period', 'invalid', 'Invalid period')
				return;
			}			
		}
		
		Wizard.setInputStatus('period', 'valid');
		periodIsValid = true;
	}

	
	report.validateStep1 = function(event)
	{
		if (!periodIsValid) event.preventDefault();
	}
	
	
	report.submit = function(event)
	{
		var frm = document.forms['reggie'];
		
		var url = '../ScanBQuarterMonthReport.servlet?ID='+App.getSessionId();
		url += '&cmd=scanbquartermonthreport';
		url += '&fdate='+encodeURIComponent(frm.fromdate.value);
		url += '&tdate='+encodeURIComponent(frm.todate.value);
		url += '&vtype='+encodeURIComponent(frm.viewtype.value);
		url += '&site='+encodeURIComponent(frm.sites.value);
		url += '&cvariant='+encodeURIComponent(frm.chartvariant.value);
		
		Wizard.showLoadingAnimation('Generating plots...');
		request = Wizard.asyncJsonRequest(url, report.onPlotGenerated);
	}

	report.onPlotGenerated = function(response)
	{
		request = null;
		var frm = document.forms['reggie'];		

		var plotReport = response.report;
		var reportTable;
		var reportDiv = Doc.element('reportdiv');
		
		var plots = plotReport.plotStatistics;
		// Draw plots
		for (var plotNo = 0; plotNo < plots.length; plotNo++)
		{
			// Get plot JSON data container with extra info and plot data
			var plot = plots[plotNo];

			// Get plot JSON extra info and plot data
			var plotChartVariant = plot.chartVariant;
			var plotViewType = plot.viewType;
			var plotOptionalHeadline = plot.optionalHeadline;
			var boxPlotJsonData = plot.plotData;

			// Create plot from plot JSON data
			if (plotOptionalHeadline != null && plotOptionalHeadline != '')
			{
				// Print optional headline
				var headlineText = document.createElement('text');
				headlineText.innerHTML = "<BR>" + plotOptionalHeadline + "<BR>";
				reportDiv.appendChild(headlineText);
			}

			var plotKey = plotChartVariant + '_' + plotViewType;
/*
					// Add plot key as debug text
					var debugText = document.createElement('text');
					debugText.innerHTML = "<BR>" + plotKey + "<BR>";
					reportDiv.appendChild(spacer);
					reportDiv.appendChild(debugText);
*/
			// Add plot
			var canvasInTable = document.createElement('canvas');
			canvasInTable.setAttribute('id', plotKey);
			canvasInTable.setAttribute('width', PLOT_WIDTH * PLOT_SCALE);
			canvasInTable.setAttribute('height', PLOT_HEIGHT * PLOT_SCALE);
			createBoxPlot(boxPlotJsonData, canvasInTable, PLOT_WIDTH, PLOT_HEIGHT, PLOT_SCALE);
			reportDiv.appendChild(canvasInTable);					
			canvasInTable.addEventListener('mousemove', report.drawCanvasGuidelines, false);
			canvasInTable.addEventListener('mouseleave', report.hideCanvasGuidelines, false);
		}
		
		// Print optional appended info
		var appendedInfo = plotReport.appendedInfo;
		if (appendedInfo != null && appendedInfo != '')
		{
			// Print optional headline
			var infoText = document.createElement('text');
			infoText.innerHTML = "<BR>" + appendedInfo + "<BR>";
			reportDiv.appendChild(infoText);
		}
		
		var caseSummary = reportDiv.getElementsByClassName('case-summary');
		for (var i = 0; i < caseSummary.length; i++)
		{
			var cs = caseSummary[i];
			Events.addEventHandler(cs, 'click', report.showCaseSummary);
		}
		
		Doc.show('step-2');
		Doc.show('printButton');
		Doc.show('gorestart');		

	}
	
	report.drawCanvasGuidelines = function(event)
	{
		if (!xGuide)
		{
			xGuide = Doc.element('x-guide');
			yGuide = Doc.element('y-guide');
		}
		var canvas = event.target;
		var centerX = event.clientX;
		var centerY = event.clientY;
		var canvasPos = Doc.getElementPosition(canvas);
		var startX = canvasPos.left+50;
		var widthX = canvasPos.width-100;
		var startY = canvasPos.top+80;
		var heightY = canvasPos.height-160;
		if (centerY < startY || centerY > startY+heightY)
		{
			// Hide when mouse is in the text area below or over the plot
			yGuide.style.display = 'none';
		}
		else
		{
			yGuide.style.top = (centerY-2) + 'px';
			yGuide.style.left = startX + 'px';
			yGuide.style.width = widthX + 'px';
			yGuide.style.display = 'block';
		}
		if (centerX < startX || centerX > startX + widthX)
		{
			// Hide when mouse is in the text area to the left or right of the plot
			xGuide.style.display = 'none';
		}
		else
		{
			// Avoid drawing outside the avilable hight (causes an extra pair of scroll bars)
			var yLimit = App.getWindowPosition().height;
			if (startY+heightY > yLimit)
			{
				heightY = yLimit-startY;
			}
			xGuide.style.top = startY + 'px';
			xGuide.style.left = (centerX-2) + 'px';
			xGuide.style.height = heightY + 'px';
			xGuide.style.display = 'block';
		}

	}
	
	report.hideCanvasGuidelines = function(event)
	{
		xGuide.style.display = 'none';
		yGuide.style.display = 'none';
	}

	
	report.printVersion = function()
	{
		var printNote = '<b>Note!</b> For better printing set page orientation to <i>portrait</i>.';
		printNote += ' Scale down to <i>90%</i> to fit 2 plots per page.';
		Reggie.openPrintWindow('step-2-content', 'Sample processing statistics', 'portrait', printNote, '../', 'report.css');
	}

	report.showCaseSummary = function(event)
	{
		var caseName = Data.get(event.currentTarget, 'case-name');
		if (caseName)
		{
			var url = 'case_summary.jsp?ID='+App.getSessionId();
			url += '&caseName='+encodeURIComponent(caseName);
			url += '&pageType=popup';
			Dialogs.openPopup(url, 'CaseSummary', 1000, 700);
		}
	}
	
	return report;
}();

Doc.onLoad(Report.initPage);
